# -*- coding: utf-8 -*-
"""
main program for IMRT QA PDF report parser
Created on Thu May 30 2019
@author: Dan Cutright, PhD
"""

from ..utilities import are_all_strings_in_text, get_csv
from dateutil.parser import parse as date_parser


class Delta4Report:
    def __init__(self):
        self.report_type = 'delta4'
        self.columns = ['Patient Name', 'Patient ID', 'Plan Date', 'Energy', 'Daily Corr', 'Norm Dose', 'Dev', 'DTA',
                        'Gamma-Index', 'Dose Dev', 'Radiation Dev', 'Gamma Pass Criteria', 'Gamma Dose Criteria',
                        'Gamma Dist Criteria']
        self.identifiers = ['ScandiDos AB', 'Treatment Summary', 'Acceptance Limits', 'Daily corr',
                            'Selected Detectors', 'Parameter Definitions & Acceptance Criteria, Detectors']

        self.treatment_summary_columns = ['Beam', 'Gantry', 'Energy', 'Daily Corr', 'Norm Dose',
                                          'Dev', 'DTA', 'Gamma-Index', 'Dose Dev']
        self.data = {}
        self.index_start = {}
        self.index_end = {}
        self.text = None

    def process_data(self, text_data):
        self.text = text_data.split('\n')

        # Patient information
        if 'PRE-TREATMENT REPORT' in self.text[3]:
            self.data['patient_name'] = self.text[0]
            self.data['patient_id'] = self.text[1]
        elif 'Clinic' not in self.text[2]:
            self.data['patient_name'] = self.text[2]
            self.data['patient_id'] = self.text[3]
        else:
            if 'Treatment Summary' in self.text:
                tx_sum_index = self.text.index('Treatment Summary')
                self.data['patient_name'] = self.text[tx_sum_index-3]
                self.data['patient_id'] = self.text[tx_sum_index-2]
            else:
                self.data['patient_name'] = 'Not found'
                self.data['patient_id'] = 'Not found'

        # Beam
        self.index_start['Beam'] = self.get_index_of_next_text_block(self.get_string_index_in_text('°'))
        self.index_end['Beam'] = self.get_index_of_next_text_block(self.index_start['Beam']) - 1
        if self.text[self.index_start['Beam']] == 'Gantry':
            self.index_start['Beam'] = self.get_index_of_next_text_block(self.index_end['Beam'])
            self.index_end['Beam'] = self.get_index_of_next_text_block(self.index_start['Beam']) - 1
        self.data['Beam'] = self.get_data_block('Beam')
        if 'Fraction' in self.data['Beam'][0]:
            self.data['Beam'].pop(0)

        # Gantry
        self.index_start['Gantry'] = self.get_index_of_next_text_block(self.index_end['Beam'])
        self.index_end['Gantry'] = self.get_index_of_next_text_block(self.index_start['Gantry']) - 1
        self.data['Gantry'] = ['N/A'] + self.get_data_block('Gantry')
        if 'Fraction' in self.data['Gantry']:
            self.data['Gantry'].pop(self.data['Gantry'].index('Fraction'))
        energy_override = []  # sometimes the energy is on the same line as the gantry
        for i, row in enumerate(self.data['Gantry']):
            self.data['Gantry'][i] = row.replace('\xc2', '').replace('\xb0', '')
            energy_override.append(None)
            row_split = row.split(' ')
            if len(row_split) > 3:
                energy_override[-1] = ' '.join(row_split[-2:])
                self.data['Gantry'][i] = self.data['Gantry'][i].replace(energy_override[-1], '').strip()

        # Dose and analysis
        self.index_start['Analysis'] = self.get_string_index_in_text('Daily corr Norm') + 2
        self.index_end['Analysis'] = self.get_index_of_next_text_block(self.index_start['Analysis']) - 1
        analysis_data_block = self.text[self.index_start['Analysis']:self.index_end['Analysis']]
        analysis_data = []
        while analysis_data_block:
            row = analysis_data_block.pop(0)

            if 'cGy' not in row:
                row = "%s %s" % (analysis_data_block.pop(0), row)

            if '%' not in row:
                row = "%s %s" % (row, analysis_data_block.pop(0))

            if 'cGy' in row and '%' in row:
                row = row.split('%')
                data = [row[0].split('cGy')[0].strip(),
                        row[0].split('cGy')[1].strip()]
                data.extend(row[1:-1])
                analysis_data.append(data)

        self.data['Norm Dose'] = [row[0] for row in analysis_data]
        self.data['Dev'] = [row[1].strip() for row in analysis_data]
        self.data['DTA'] = [row[2].strip() for row in analysis_data]
        self.data['Gamma-Index'] = [row[3].strip() for row in analysis_data]
        self.data['Dose Dev'] = [row[4].strip() for row in analysis_data]

        # Daily Correction Factor
        self.index_start['Daily Corr'] = self.get_index_of_next_text_block(self.index_end['Analysis'])
        if 'Det within acceptance' in self.text[self.index_start['Daily Corr']]:
            self.index_start['Daily Corr'] = self.get_index_of_next_text_block(self.index_start['Daily Corr'])
            if 'index dose dev' in self.text[self.index_start['Daily Corr']]:
                self.index_start['Daily Corr'] = self.get_index_of_next_text_block(self.index_start['Daily Corr'])
                if 'factor' in self.text[self.index_start['Daily Corr']]:
                    self.index_start['Daily Corr'] = self.get_index_of_next_text_block(self.index_start['Daily Corr'])
        self.index_end['Daily Corr'] = self.get_index_of_next_text_block(self.index_start['Daily Corr']) - 1
        self.data['Daily Corr'] = ['N/A'] + self.get_data_block('Daily Corr')
        for i, row in enumerate(self.data['Daily Corr']):
            if not row.isdigit():
                self.data['Daily Corr'][i] = row[-5:]

        # Energy
        self.index_start['Energy'] = self.get_index_of_next_text_block(self.index_end['Daily Corr'])
        if 'dose dev' in self.text[self.index_start['Energy']]:
            self.index_start['Energy'] = self.get_index_of_next_text_block(self.index_start['Energy'])
        self.index_end['Energy'] = self.get_index_of_next_text_block(self.index_start['Energy']) - 1
        self.data['Energy'] = ['N/A'] + self.get_data_block('Energy')
        if any(energy_override):  # replace values with overrides found in Gantry code block
            for i, override in enumerate(energy_override):
                if override is not None:
                    if len(self.data['Energy']) > i:
                        self.data['Energy'][i] = override

        # Gamma Criteria
        self.index_start['Gamma Criteria'] = self.text.index('Parameter Definitions & Acceptance Criteria, Detectors')
        self.index_start['Acceptance Limits'] = self.text.index('Acceptance Limits')
        self.index_end['Gamma Criteria'] = self.index_start['Acceptance Limits'] - 1
        self.index_end['Acceptance Limits'] = self.get_index_of_next_text_block(self.index_start['Acceptance Limits']) - 1

        for row in self.get_data_block('Gamma Criteria'):
            if 'mm' in row:
                temp = row.split('mm')[0].strip()
                try:
                    float(temp)
                    self.data['gamma_dist'] = temp
                except:
                    pass
            elif '±' in row:
                self.data['gamma_dose'] = row.split('±')[1].replace('%', '')

        self.data['gamma_pass'] = self.get_data_block('Acceptance Limits')[-1].split('%')[0]

    @property
    def radiation_device(self):
        for row in self.text:
            if row.startswith('Radiation Device: '):
                return row.replace('Radiation Device: ', '')
        return None

    @property
    def measured_date(self):
        index_of_first_date = self.get_index_of_first_date()
        date_candidate_1 = self.text[index_of_first_date].split(' ')[0]
        date_candidate_2 = self.text[index_of_first_date+2].split(' ')[0]
        try:
            return str(date_parser(date_candidate_1)).split(' ')[0]
        except:
            try:
                return str(date_parser(date_candidate_2)).split(' ')[0]
            except:
                pass
        return None

    def get_index_of_first_date(self):
        for i, row in enumerate(self.text):
            if are_all_strings_in_text(row, ['/', ':', 'M']) or \
                    are_all_strings_in_text(row, ['.', ':', 'M']):
                try:
                    date_parser(row.split(' ')[0].strip())
                    return i
                except:
                    pass
        return None

    def get_string_index_in_text(self, string, start_index=0):
        for i, row in enumerate(self.text[start_index:]):
            if string in row:
                return i
        return None

    def get_index_of_next_text_block(self, start_index):
        for i, row in enumerate(self.text[start_index:]):
            if row.strip() == '':
                return i + start_index + 1
        return None

    def get_data_block(self, data_type):
        return self.text[self.index_start[data_type]:self.index_end[data_type]]

    @property
    def summary_data(self):
        return {'Patient Name': self.data['patient_name'],
                'Patient ID': self.data['patient_id'],
                'Plan Date': self.measured_date,
                'Energy': '/'.join(list(set([e for e in self.data['Energy'] if e != 'N/A']))),
                'Daily Corr': sum([float(f) for f in self.data['Daily Corr'] if f != 'N/A']) / (
                            len(self.data['Daily Corr']) - 1),
                'Norm Dose': float(self.data['Norm Dose'][0]),
                'Dev': float(self.data['Dev'][0]),
                'DTA': float(self.data['DTA'][0]),
                'Gamma-Index': float(self.data['Gamma-Index'][0]),
                'Dose Dev': float(self.data['Dose Dev'][0]),
                'Radiation Dev': self.radiation_device,
                'Gamma Pass Criteria': float(self.data['gamma_pass']),
                'Gamma Dose Criteria': float(self.data['gamma_dose']),
                'Gamma Dist Criteria': float(self.data['gamma_dist'])}

    @property
    def csv(self):
        return get_csv(self.summary_data, self.columns)



# -*- coding: utf-8 -*-
"""
The following code is from StackOverflow
https://stackoverflow.com/questions/26494211/extracting-text-from-a-pdf-file-using-pdfminer-in-python
Web page accessed on May 30, 2019
"""

from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
from pdfminer.converter import TextConverter
from pdfminer.layout import LAParams
from pdfminer.pdfpage import PDFPage
from io import StringIO


def convert_pdf_to_txt(path):
    rsrcmgr = PDFResourceManager()
    retstr = StringIO()
    codec = 'utf-8'
    laparams = LAParams()
    device = TextConverter(rsrcmgr, retstr, codec=codec, laparams=laparams)
    fp = open(path, 'rb')
    interpreter = PDFPageInterpreter(rsrcmgr, device)
    password = ""
    maxpages = 0
    caching = True
    pagenos = set()

    for page in PDFPage.get_pages(fp, pagenos, maxpages=maxpages, password=password, caching=caching,
                                  check_extractable=True):
        interpreter.process_page(page)

    text = retstr.getvalue()

    fp.close()
    device.close()
    retstr.close()
    return text



# -*- coding: utf-8 -*-
"""
ArcCheck Report class
Created on Fri Jun 21 2019
@author: Dan Cutright, PhD
@contributor: Marc J.P. Chamberland, PhD
"""

from ..utilities import get_csv
import re


class ArcCheckReport:
    def __init__(self):
        self.report_type = 'arccheck'
        self.columns = ['Patient Last Name', 'Patient First Name', 'Patient ID', 'Plan Date', 'Dose Type', 'Difference (%)', 'Distance (mm)',
                        'Threshold (%)', 'Meas Uncertainty', 'Analysis Type', 'Total Points', 'Passed', 'Failed',
                        '% Passed', 'Min', 'Max', 'Average', 'Std Dev', 'X offset (mm)', 'Y offset (mm)', 'Notes']
        self.identifiers = ['QA File Parameter', 'Threshold', 'Notes', 'Reviewed By :', 'SSD', 'Depth', 'Energy', 'ArcCHECK']
        self.text = None
        self.data = {}

    def process_data(self, text_data):
        self.text = text_data.split('\n')
        self.data['date'], self.data['hospital'] = [], []
        for row in self.text:
            if row.find('Date: ') > -1:
                self.data['date'] = row.strip('Date: ')
            if row.find('Hospital Name: ') > -1:
                self.data['hospital'] = row.split('Hospital Name: ', 1)[-1]

            if self.data['date'] and self.data['hospital']:
                break

        self.data['qa_file_parameter'] = self.get_group_results('QA File Parameter')

        x_offset = '0'
        y_offset = '0'
        try:
            plan_index = self.text.index('Plan')
            if self.text[plan_index + 2].find('CAX') > -1:
                x_offset, y_offset = re.findall(r'[-+]?[.]?[\d]+(?:,\d\d\d)*[\.]?\d*(?:[eE][-+]?\d+)?',
                                                self.text[plan_index + 2])
        except ValueError:
            pass

        self.data['cax_offset'] = {'X offset': str(x_offset), 'Y offset': str(y_offset)}

        # Dose Comparison Block
        try:
            self.text.index('Absolute Dose Comparison')
            self.data['dose_comparison_type'] = 'Absolute Dose Comparison'
        except ValueError:
            self.data['dose_comparison_type'] = 'Relative Comparison'
        self.data['dose_comparison'] = self.get_group_results(self.data['dose_comparison_type'])
        if '% Diff' in list(self.data['dose_comparison']):  # Alternate for Difference (%) for some versions of report?
            self.data['dose_comparison']['Difference (%)'] = self.data['dose_comparison']['% Diff']
        if 'Threshold' in list(self.data['dose_comparison']):  # Alternate for Threshold (%) for some versions of report?
            self.data['dose_comparison']['Threshold (%)'] = self.data['dose_comparison']['Threshold']

        # Summary Analysis Block
        try:
            self.text.index('Summary (Gamma Analysis)')
            self.data['analysis_type'] = 'Gamma'
        except ValueError:
            try:
                self.data['analysis_type'] = 'DTA'
            except ValueError:
                self.data['analysis_type'] = 'GC'  # Gradient Correction

        self.data['summary'] = self.get_group_results('Summary (%s Analysis)' % self.data['analysis_type'])

        # Gamma Index Summary Block
        try:
            self.text.index('Gamma Index Summary')
            self.data['gamma_stats'] = 'Gamma Index Summary'
        except ValueError:
            self.data['gamma_stats'] = {'Minimum': 'n/a',
                                'Maximum': 'n/a',
                                'Average': 'n/a',
                                'Stdv': 'n/a'}

        self.data['gamma_stats'] = self.get_gamma_statistics(self.data['gamma_stats'])

        self.data['notes'] = self.text[self.text.index('Notes') + 1]

    def get_gamma_statistics(self, stats_delimiter):
        gamma_stats = {}
        stats_fields = ['Minimum', 'Maximum', 'Average', 'Stdv']

        group_start = self.text.index(stats_delimiter)

        for field in stats_fields:
            field_start = self.text[group_start:-1].index(field) + 1
            gamma_stats[field] = self.text[group_start:-1][field_start]

        return gamma_stats

    def get_group_results(self, data_group):
        """
        ArcCheck reports contain three blocks of results. data_group may be among the following:
            'QA File Parameter'
            'Absolute Dose Comparison' or 'Relative Comparison'
            'Gamma' or 'DTA'
        """
        group_start = self.text.index(data_group)
        var_name_start = group_start + 1
        data_start = self.text[var_name_start:-1].index('') + 1 + var_name_start
        data_count = data_start - var_name_start

        # If patient name is too long, sometimes the pdf parsing gets off-set
        if self.text[data_start] == 'Set1':
            data_start += 1

        group_results = {}
        for i in range(data_count):
            if self.text[var_name_start+i]:
                group_results[self.text[var_name_start+i]] = self.text[data_start+i].replace(' : ', '')

        return group_results

    @property
    def summary_data(self):
        """
        Collect the parsed data into a dictionary with keys corresponding to columns
        :return: parsed data
        :rtype: dict
        """
        patient_name = self.data['qa_file_parameter']['Patient Name'].replace('^', ' ').split(', ')
        if len(patient_name) > 1:
            last_name = patient_name[0].title()
            first_name = patient_name[1].title()
        elif len(patient_name) == 1:
            last_name = patient_name[0].title()
            first_name = 'n/a'
        else:
            last_name = 'n/a'
            first_name = 'n/a'

        return {'Patient Last Name': last_name,
                'Patient First Name': first_name,
                'Patient ID': self.data['qa_file_parameter']['Patient ID'],
                'Plan Date': self.data['qa_file_parameter']['Plan Date'],
                'Dose Type': self.data['dose_comparison_type'],
                'Difference (%)': self.data['dose_comparison']['Difference (%)'],
                'Distance (mm)': self.data['dose_comparison']['Distance (mm)'],
                'Threshold (%)': self.data['dose_comparison']['Threshold (%)'],
                'Meas Uncertainty': self.data['dose_comparison']['Meas Uncertainty'],
                'Analysis Type': self.data['analysis_type'],
                'Total Points': self.data['summary']['Total Points'],
                'Passed': self.data['summary']['Passed'],
                'Failed': self.data['summary']['Failed'],
                '% Passed': self.data['summary']['% Passed'],
                'Min': self.data['gamma_stats']['Minimum'],
                'Max': self.data['gamma_stats']['Maximum'],
                'Average': self.data['gamma_stats']['Average'],
                'Std Dev': self.data['gamma_stats']['Stdv'],
                'X offset (mm)': self.data['cax_offset']['X offset'],
                'Y offset (mm)':self.data['cax_offset']['Y offset'],
                'Notes': self.data['notes']}

    @property
    def csv(self):
        return get_csv(self.summary_data, self.columns)



# -*- coding: utf-8 -*-
"""
main program for IMRT QA PDF report parser
Created on Thu May 30 2019
@author: Dan Cutright, PhD
"""
from os.path import isdir, isfile, join
from os import walk, listdir
from datetime import datetime
from .parsers.parser import ReportParser
from .utilities import DELIMITER
import argparse
from pathvalidate import sanitize_filename
import sys


CURRENT_VERSION = '0.2.7'


def convert_pdf_to_txt(abs_file_path):
    if sys.version_info[0] < 3:
        from .pdf_to_text_python2 import convert_pdf_to_txt as func
        return func(abs_file_path)
    else:
        from .pdf_to_text import convert_pdf_to_txt as func
        return func(abs_file_path)


def pdf_to_qa_result(abs_file_path):
    """
    Given an absolute file path, convert file to text
    :param abs_file_path: file to be converted to text
    :return: csv row to be written to csv file, report type, column headers for csv
    :rtype: tuple
    """

    try:
        text = convert_pdf_to_txt(abs_file_path)
    except Exception as e:
        print("Non-compatible PDF detected: %s" % abs_file_path)
        print(str(e))
        return ''

    report_obj = ReportParser(text)
    if report_obj.report is not None:
        return report_obj.csv + DELIMITER + abs_file_path, report_obj.report_type, report_obj.columns


def process_files(init_directory, ignore_extension=False, output_file=None, output_dir=None, no_recursive_search=False):
    """
    Given an initial directory, process all pdf files into parser classes, write their csv property to results_file
    :param init_directory: initial scanning directory
    :param ignore_extension: if you'd like to catch pdf files that are missing .pdf extension, set to True
    :type ignore_extension: bool
    :param output_file: user specified output file name, report type will be prepended to this value
    :param output_dir: user specified output directory, default value is to local directory
    :param no_recursive_search: to ignore sub-directories, set to True
    :type no_recursive_search: bool
    """

    time_stamp = str(datetime.now()).replace(':', '-').replace('.', '-')
    if output_file is None:
        output_file = "results_%s.csv" % time_stamp

    if no_recursive_search:
        for file_name in listdir(init_directory):
            if ignore_extension or file_name.lower().endswith('.pdf'):
                file_path = join(init_directory, file_name)
                process_file(file_path, output_file, output_dir)
    else:
        for dirName, _, fileList in walk(init_directory):  # iterate through files and all sub-directories
            for fileName in fileList:
                if ignore_extension or fileName.lower().endswith('.pdf'):
                    file_path = join(dirName, fileName)
                    process_file(file_path, output_file, output_dir)


def process_file(file_path, output_file, output_dir):
    try:
        row, report_type, columns = pdf_to_qa_result(file_path)  # process file
        current_file = "%s_%s" % (report_type, output_file)  # prepend report type to file name
        if output_dir:
            current_file = join(output_dir, current_file)
        if row:
            if not isfile(current_file):  # if file doesn't exist, need to write columns
                with open(current_file, 'w') as csv:
                    csv.write(DELIMITER.join(columns) + '\n')
            with open(current_file, "a") as csv:  # write the processed data
                csv.write(row + '\n')
            print("Processed: %s" % file_path)
    except Exception as e:  # error likely occurred in pdf_to_qa_result()
        print("Non-compatible PDF detected: %s" % file_path)
        print(str(e))


def main():

    cmd_parser = argparse.ArgumentParser(description="Command line interface for IQDM")
    cmd_parser.add_argument('-ie', '--ignore-extension',
                            dest='ignore_extension',
                            help='Script will check all files, not just ones with .pdf extensions',
                            default=False,
                            action='store_true')
    cmd_parser.add_argument('-od', '--output-dir',
                            dest='output_dir',
                            help='Output stored in local directory by default, specify otherwise here',
                            default=None)
    cmd_parser.add_argument('-of', '--output-file',
                            dest='output_file',
                            help='Output will be saved as <report_type>_results_<time-stamp>.csv by default. '
                                 'Define this tag to customize file name after <report_type>_',
                            default=None)
    cmd_parser.add_argument('-ver', '--version',
                            dest='print_version',
                            help='Print the IQDM version',
                            default=False,
                            action='store_true')
    cmd_parser.add_argument('--no-recursive-search',
                            dest='no_recursive_search',
                            help='Include this flag to skip sub-directories',
                            default=False,
                            action='store_true')
    cmd_parser.add_argument('initial_directory', nargs='?',
                            help='Scan this directory and all sub-directories')
    args = cmd_parser.parse_args()

    # if args.initial_directory and len(args.initial_directory) > 2:
    #     print("Too many arguments provided. Please only provide the initial scanning directory after IQDM")
    #     return

    if not args.initial_directory or len(args.initial_directory) < 2:
        if args.print_version:
            print('IMRT-QA-Data-Miner: IQDM v%s' % CURRENT_VERSION)
            return
        else:
            print('Initial directory not provided!')
            return

    if not isdir(args.initial_directory):
        print("%s is not a valid or accessible directory" % args.initial_directory)
        return

    output_file, print_file_name_change = None, False
    if args.output_file:
        output_file = sanitize_filename(args.output_file)
        if output_file not in args.output_file:
            print_file_name_change = True

    process_files(args.initial_directory,
                  ignore_extension=args.ignore_extension,
                  output_file=output_file,
                  output_dir=args.output_dir,
                  no_recursive_search=args.no_recursive_search)

    if args.print_version:
        print('IMRT-QA-Data-Miner: IQDM v%s' % CURRENT_VERSION)

    if print_file_name_change:
        print('Output file name was changed to <report_type>_%s' % output_file)


if __name__ == '__main__':
    main()



# The following code is from StackOverflow
# https://stackoverflow.com/questions/26494211/extracting-text-from-a-pdf-file-using-pdfminer-in-python
# Answer given by DuckPuncher
# Web page accessed on April 18, 2018

from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
from pdfminer.converter import TextConverter
from pdfminer.layout import LAParams
from pdfminer.pdfpage import PDFPage
from cStringIO import StringIO


def convert_pdf_to_txt(path):
    rsrcmgr = PDFResourceManager()
    retstr = StringIO()
    codec = 'utf-8'
    laparams = LAParams()
    device = TextConverter(rsrcmgr, retstr, codec=codec, laparams=laparams)
    fp = file(path, 'rb')
    interpreter = PDFPageInterpreter(rsrcmgr, device)
    password = ""
    maxpages = 0
    caching = True
    pagenos=set()

    for page in PDFPage.get_pages(fp, pagenos, maxpages=maxpages, password=password,caching=caching, check_extractable=True):
        interpreter.process_page(page)

    text = retstr.getvalue()

    fp.close()
    device.close()
    retstr.close()
    return text



# -*- coding: utf-8 -*-
"""
main program for IMRT QA PDF report parser
Created on Thu May 30 2019
@author: Dan Cutright, PhD
"""


DELIMITER = ','  # delimiter for the csv output file for process_files
ALTERNATE = '^'  # replace the delimiter character with this so not to confuse csv file parsing


def are_all_strings_in_text(text, list_of_strings):
    """
    :param text: output from convert_pdf_to_text
    :type text: list of str
    :param list_of_strings: a list of strings used to identify document type
    :type list_of_strings: list of str
    :return: Will return true if every string in list_of_strings is found in the text data
    :rtype: bool
    """
    for str_to_find in list_of_strings:
        if str_to_find not in text:
            return False
    return True


def get_csv(data, columns):
    """
    Convert a dictionary of data into a row for a csv file
    :param data: a dictionary with values with str representations
    :type data: dict
    :param columns: a list of keys dictating the order of the csv
    :type columns: list
    :return: a csv string delimited by DELIMITER
    :rtype: str
    """
    clean_csv = [str(data[column]).replace(DELIMITER, ALTERNATE) for column in columns]
    return DELIMITER.join(clean_csv)



# -*- coding: utf-8 -*-
"""
main program for IMRT QA PDF report parser
Created on Thu May 30 2019
@author: Dan Cutright, PhD
"""

from ..utilities import are_all_strings_in_text
from .arccheck import ArcCheckReport
from .mapcheck import MapcheckReport
from .delta4 import Delta4Report


# These classes will be checked in ReportParser.get_report()
REPORT_CLASSES = [ArcCheckReport, MapcheckReport, Delta4Report]


class ReportParser:
    """
    This class determines which Report class to use and subsequently processes the data.

    Use of this class requires each report class listed in REPORT_CLASSES contains the following properties:
        identifiers:    this is a list of strings that collectively are uniquely found in a report type
        columns:        a list of strings indicating the columns of the csv to be output
        csv:            a string of values for each column, delimited with DELIMITER in utilities.py
        report_type:    a string describing the report, this will be used in the results filename created in main.py

    This class also requires the following method:
        process_data(text_data):    processing the data does not occur until this is called

    If ReportParser.report is None, the input text was not identified to be any of the report classes listed in
    REPORT_CLASSES
    """
    def __init__(self, text):
        self.report = self.get_report(text)
        if self.report:
            self.columns = self.report.columns
            self.csv = self.report.csv
            self.report_type = self.report.report_type

    @staticmethod
    def get_report(text):
        for report_class in REPORT_CLASSES:
            rc = report_class()  # initialize class to access identifiers
            if are_all_strings_in_text(text, rc.identifiers):
                rc.process_data(text)  # parse the text data
                return rc
        return None



from setuptools import setup, find_packages

requires = [
    'pdfminer.six',
    'pdfminer',
    'numpy',
    'python-dateutil',
    'chardet == 3.0.4',
    'pathvalidate'
]

setup(
    name='IQDM',
    include_package_data=True,
    packages=find_packages(),
    version='0.2.8',
    description='Scans a directory for IMRT QA results',
    author='Dan Cutright',
    author_email='dan.cutright@gmail.com',
    url='https://github.com/cutright/IMRT-QA-Data-Miner/',
    download_url='https://github.com/cutright/IMRT-QA-Data-Miner/archive/master.zip',
    license="MIT License",
    keywords=['radiation therapy', 'qa', 'research'],
    classifiers=[],
    install_requires=requires,
    entry_points={
        'console_scripts': [
            'IQDM=IQDM.main:main',
        ],
    },
    long_description="""IMRT QA Data Miner
    
    This software will iteratively scan all files with in a directory to extract data from IMRT QA reports 
    and generate a csv file.
    """
)